use crate::{Data, Error};
use clickhouse_pool::traits::Model;
use database::{
    trivial::{Trivial, TrivialStatus},
    trivial_question::TrivialQuestion,
    trivial_round::TrivialRound,
};
use poise::serenity_prelude::{self as serenity, ActivityData, CreateMessage, FullEvent};
use tracing::{info, span, Instrument};

pub async fn event_handler(
    ctx: &serenity::Context,
    event: &serenity::FullEvent,
    _framework: poise::FrameworkContext<'_, Data, Error>,
    data: &Data,
) -> Result<(), Error> {
    match event {
        serenity::FullEvent::Ready { data_about_bot, .. } => {
            span!(tracing::Level::INFO, "event_handler", event = "Ready").in_scope(|| {
                info!("Logged in as {}", data_about_bot.user.name);
                info!("Guilds: {}", data_about_bot.guilds.len());
                ctx.set_activity(Some(ActivityData {
                    name: data.config.default_status.clone(),
                    kind: serenity::ActivityType::Playing,
                    url: None,
                    state: None,
                }));
                info!(
                    "Bot is ready and set to default status: {}",
                    data.config.default_status
                );
            });
        }
        FullEvent::Message { new_message, .. } => {
            if new_message.author.bot || new_message.guild_id.is_none() {
                return Ok(());
            }
            let guild_id = match new_message.guild_id {
                Some(id) => Some(id),
                None => None,
            };

            let channel_id = new_message.channel_id;

            let manager_pool = data.datalake_config.clone();

            let trivia_query = Trivial::build_select_query(
                Some(&format!(
                    "guild_id = {} and channel_id = {}",
                    guild_id.unwrap().get(),
                    channel_id.get()
                )),
                None,
                None,
            );
            let trivia_games: Vec<Trivial> = manager_pool
                .execute_select_with_retry(&trivia_query)
                .await?;
            if trivia_games.is_empty() {
                return Ok(());
            }
            let trivia = &trivia_games[0];
            if trivia.status != TrivialStatus::Started {
                return Ok(());
            }
            let trivia_round_query = TrivialRound::build_select_query(
                Some(&format!(
                    "trivial_id = '{}' and finished = false",
                    trivia.id
                )),
                None,
                None,
            );
            let trivia_rounds: Vec<TrivialRound> = manager_pool
                .execute_select_with_retry(&trivia_round_query)
                .await?;
            if trivia_rounds.is_empty() {
                return Ok(());
            }

            return async {
                let mut trivia_round = trivia_rounds[0].clone();
                let trivia_question_query = TrivialQuestion::build_select_query(
                    Some(&format!("id = '{}'", trivia_round.question_id)),
                    None,
                    None,
                );
                let trivia_questions: Vec<TrivialQuestion> = manager_pool
                    .execute_select_with_retry(&trivia_question_query)
                    .await?;
                if trivia_questions.is_empty() {
                    return Ok(());
                }
                let trivia_question = &trivia_questions[0];
                let answer = trivia_question.answer.to_lowercase();
                let mutate_round = match trivia.reward_kind {
                    database::trivial::TrivialRewardKind::TopThree => {
                        trivia_round.answer.len() < 3
                            && trivia_round
                                .answer
                                .iter()
                                .filter(|&&(id, _, _)| new_message.author.id.get() == id)
                                .count()
                                == 0
                    }
                    database::trivial::TrivialRewardKind::TopFive => {
                        trivia_round.answer.len() < 5
                            && trivia_round
                                .answer
                                .iter()
                                .filter(|&&(id, _, _)| new_message.author.id.get() == id)
                                .count()
                                == 0
                    }
                    database::trivial::TrivialRewardKind::OnlyTheFirstOne => {
                        trivia_round.answer.is_empty()
                    }
                };
                if !mutate_round || !new_message.content.contains(&answer) {
                    return Ok(());
                }
                let mut inserter = manager_pool
                    .get_insert::<TrivialRound>(TrivialRound::table_name())
                    .await?;
                trivia_round.sign = -1; // Set sign to -1 to indicate an update
                inserter.write(&trivia_round.clone()).await?;
                trivia_round.sign = 1; // Set sign to 1 to indicate a new answer
                trivia_round.answer.push((
                    new_message.author.id.get(),
                    new_message.content.clone(),
                    *new_message.timestamp,
                ));
                inserter.write(&trivia_round.clone()).await?;
                let optimize = TrivialRound::optimize_table_sql();
                match manager_pool.execute_with_retry(&optimize).await {
                    Ok(_) => {}
                    Err(e) => {
                        tracing::error!("Failed to optimize trivia round table: {}", e);
                    }
                }
                let builder = CreateMessage::new()
                    .content("Answer taken into account!")
                    .reference_message(new_message)
                    .flags(serenity::MessageFlags::EPHEMERAL);
                match channel_id.send_message(&ctx.http, builder).await {
                    Ok(msg) => {
                        // sleep for a short duration to ensure the message is sent
                        tokio::time::sleep(std::time::Duration::from_secs(5)).await;
                        msg.delete(ctx).await.unwrap_or_else(|e| {
                            tracing::error!("Failed to delete ephemeral message: {}", e);
                        });
                    }
                    Err(e) => {
                        tracing::error!("Failed to send message: {}", e);
                    }
                }
                Ok(())
            }
            .instrument(span!(
                tracing::Level::INFO,
                "event_handler",
                event = "Message"
            ))
            .await;
        }
        _ => {}
    }
    Ok(())
}
